unit AIPlayer;

interface

uses Player, MapData, Communication, System.Generics.Collections;

type
  TAIPlayer = class(TPlayer)
    private
      FWay: TList<TCommDirection>;
    public
      constructor Create(MD: TMapData);
      destructor Destroy; override;

      procedure HandleKey(Key: Word); override;
      procedure OnUpdate; override;

      function ComputeDistance(X: Integer; Y: Integer): Integer;
      function RandomDir: TCommDirection;
      function GetDirection(X: Integer; Y: Integer): TCommDirection;
      function ToggleDir(Dir: TCommDirection): TCommDirection;
  end;

implementation

constructor TAIPlayer.Create(MD: TMapData);
begin
  // Vytvoen listu pro zpten cestu
  inherited Create(MD);
  FWay := TList<TCommDirection>.Create;
end;

destructor TAIPlayer.Destroy;
begin
  // Vymazn listu
  FWay.Destroy;
end;

procedure TAIPlayer.HandleKey(Key: Word);
begin
end;

procedure TAIPlayer.OnUpdate;
var i, j, bestI, bestJ, distance, bestDistance, homeX, homeY, px, py: Integer;
    hasBest, hasTreasure: Boolean;
    dir: TCommDirection;
begin
  // Tato procedura se pokus nastavit cestu k zkladn nebo k nejblimu pokladu
  hasTreasure := FMapData.Comm.Player.Current <> 0;
  FMapData.GetHomeXY(homeX, homeY);

  px := FMapData.Comm.Player.X;
  py := FMapData.Comm.Player.Y;

  if (homeX = px) and (homeY = py) then
    FWay.Clear;

  if hasTreasure then
  begin
    if FWay.Count > 0 then
    begin
      dir := FWay.Last;
      FWay.Delete(FWay.Count - 1);
      dir := ToggleDir(dir);
    end else
    begin
      if px + 1 = homeX then
        dir := cdRight
      else if px - 1 = homeX then
        dir := cdLeft
      else if py + 1 = homeY then
        dir := cdDown
      else dir := cdUp;
    end;
  end else
  begin
    hasBest := False;
    for i := 1 to 10 do
      for j := 1 to 10 do
      begin
        if FMapData.IsTreasureAt(i, j) then
        begin
          distance := ComputeDistance(i, j);
          if not hasBest or (bestDistance > distance) then
          begin
            bestI := i;
            bestJ := j;
            bestDistance := distance;
            hasBest := True;
          end;
        end;
      end;
      if not hasBest then
      begin
        dir := RandomDir;
      end else
      begin
        dir := GetDirection(bestI, bestJ);
      end;
  end;

  FMapData.Move(dir);
end;

function TAIPlayer.ComputeDistance(X: Integer; Y: Integer): Integer;
var px, py: Integer;
begin
  // Vypot vzdlenost mezi hrem a danm bodem
  px := FMapData.Comm.Player.X;
  py := FMapData.Comm.Player.Y;

  Result := Abs(px - X) + Abs(py - Y);
end;

function TAIPlayer.RandomDir: TCommDirection;
var x, y: Integer;
begin
  // Vrt nhodn smr
  x := FMapData.Comm.Player.X;
  y := FMapData.Comm.Player.Y;

  if FMapData.CanMoveToPosition(x, y - 1) then
    Result := cdDown
  else if FMapData.CanMoveToPosition(x, y + 1) then
    Result := cdUp
  else if FMapData.CanMoveToPosition(x - 1, y) then
    Result := cdLeft
  else if FMapData.CanMoveToPosition(x + 1, y) then
    Result := cdRight
  else
    Result := cdUp;
end;

function TAIPlayer.GetDirection(X: Integer; Y: Integer): TCommDirection;
var px, py: Integer;
begin
  // Vrt smr k danmu bodu
  px := FMapData.Comm.Player.X;
  py := FMapData.Comm.Player.Y;

  Result := cdNone;

  if Abs(px - X) < Abs(py - Y) then
  begin
    if (py > Y) and FMapData.CanMoveToPosition(px, py - 1) then
      Result := cdUp
    else if (py < Y) and FMapData.CanMoveToPosition(px, py + 1) then
      Result := cdDown
  end
  else
  begin
    if (px < X) and FMapData.CanMoveToPosition(px + 1, py) then
      Result := cdRight
    else if (px > X) and FMapData.CanMoveToPosition(px - 1, py) then
      Result := cdLeft;
  end;

  if Result = cdNone then
  begin
    if Abs(px - X) < Abs(py - Y) then
    begin
      if (px <= X) and FMapData.CanMoveToPosition(px + 1, py) then
        Result := cdRight
      else if (px > X) and FMapData.CanMoveToPosition(px - 1, py) then
        Result := cdLeft
    end
    else
    begin
      if (py >= Y) and FMapData.CanMoveToPosition(px, py - 1) then
        Result := cdUp
      else if (py < Y) and FMapData.CanMoveToPosition(px, py + 1) then
        Result := cdDown;
    end;
  end;
  FWay.Add(Result);
end;

function TAIPlayer.ToggleDir(Dir: TCommDirection): TCommDirection;
begin
  // Oto smr
  if dir = cdUp then
    Result := cdDown
  else if dir = cdDown then
    Result := cdUp
  else if dir = cdLeft then
    Result := cdRight
  else
    Result := cdLeft;
end;

end.

