unit Communication;

interface

uses IdHTTP, System.SysUtils, System.Classes;

type
  TCommDirection = (cdNone, cdLeft, cdRight, cdUp, cdDown);

  TCommPlayerInfo = record
    X, Y, Total, Current: Integer;
  end;

  TComm = class
    private
      FConn: TIdHTTP;
      FGameID: Integer;
      FStream: TMemoryStream;
      FData: string;
      FLineData: TArray<string>;
      FAddr: string;
      FMovesLeft: Integer;
      FPlayer, FServerPlayer: TCommPlayerInfo;
      FGameEnd, FHasServerPlayer: Boolean;
      FGameResultStr: string;

      procedure StreamToString;

    public
      constructor Create;
      destructor Destroy; override;

      procedure StartGame;
      procedure SendMove(Dir: TCommDirection);

      property ServerAddress: string read FAddr write FAddr;
      property Data: string read FData;
      property LineData: TArray<string> read FLineData;
      property GameID: Integer read FGameID;
      property HasServerPlayer: Boolean read FHasServerPlayer;
      property Player: TCommPlayerInfo read FPlayer;
      property ServerPlayer: TCommPlayerInfo read FServerPlayer;
      property MovesLeft: Integer read FMovesLeft;
      property GameEnd: Boolean read FGameEnd;
      property GameResult: string read FGameResultStr;
      var OnUpdate: procedure of object;
  end;

implementation

constructor TComm.Create;
begin
  FAddr := 'http://127.0.0.1:8000/';
  FConn := TIdHTTP.Create(nil);
  FStream := TMemoryStream.Create;
end;

destructor TComm.Destroy;
begin
  FStream.Destroy;
  FConn.Destroy;
end;

procedure TComm.SendMove(Dir: TCommDirection);
var dirS: string;
begin
  case Dir of
    cdNone: dirS := 'X';
    cdLeft: dirS := 'L';
    cdRight: dirS := 'P';
    cdUp: dirS := 'N';
    cdDown: dirS := 'D';
  end;

  if dirS <> 'X' then
  begin
    try
      FConn.Get(FAddr + 'tah?id=' + IntToStr(FGameID) + '&pohyb=' + dirS, FStream);
      StreamToString;
    finally
    end;
  end
  else
  begin
    if Assigned(OnUpdate) then
      OnUpdate;
  end;
end;

procedure TComm.StartGame;
begin
  try
    FGameEnd := False;
    FConn.Get(FAddr + 'start', FStream);
    StreamToString;
  finally
  end;
end;

function parseInt(const s: String): Integer;
begin
  try
    Result := StrToInt(s);
  except
    on E: EConvertError do Result := -1;
  end;
end;

procedure TComm.StreamToString;
var playerInfo: TArray<string>;
begin
  SetString(FData, PAnsiChar(FStream.Memory), FStream.Size);
  FData := FData.Replace(Char($A), '');
  FLineData := FData.Split([Char(13)]);
  if parseInt(FLineData[0]) = -1 then
  begin
    FGameEnd := True;
    FGameResultStr := FLineData[0];
  end else
  begin
    FGameID := StrToInt(FLineData[0]);
    FMovesLeft := StrToInt(FLineData[1]);

    playerInfo := FLineData[12].Split([' ']);
    FPlayer.X := StrToInt(playerInfo[0]);
    FPlayer.Y := StrToInt(playerInfo[1]);
    FPlayer.Current := StrToInt(playerInfo[2]);
    FPlayer.Total := StrToInt(playerInfo[3]);

    if FLineData[13] <> '' then
    begin
      playerInfo := FLineData[13].Split([' ']);
      FServerPlayer.X := StrToInt(playerInfo[0]);
      FServerPlayer.Y := StrToInt(playerInfo[1]);
      FServerPlayer.Current := StrToInt(playerInfo[2]);
      FServerPlayer.Total := StrToInt(playerInfo[3]);
      FHasServerPlayer := True;
    end else FHasServerPlayer := False;
  end;
  FStream.Clear;

  if Assigned(OnUpdate) then
    OnUpdate;
end;

end.
