unit MainWindow;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Renderer, Communication, MapData, Player,
  AIPlayer, Vcl.ExtCtrls,
  Vcl.Menus, Vcl.ComCtrls, Vcl.StdCtrls, System.Threading;

type
  TMainWindowClass = class(TForm)
    Updater: TTimer;
    MainMenu1: TMainMenu;
    Soubor1: TMenuItem;
    Novhra1: TMenuItem;
    Nastavitadresuserveru1: TMenuItem;
    Konec1: TMenuItem;
    Image1: TImage;
    GroupBox1: TGroupBox;
    cpuBox: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    cpuCurrent: TLabel;
    cpuTotal: TLabel;
    playerTotal: TLabel;
    playerCurrent: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    useAI: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Label3: TLabel;
    movesLeft: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure UpdaterTimer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure Konec1Click(Sender: TObject);
    procedure Nastavitadresuserveru1Click(Sender: TObject);
    procedure Novhra1Click(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
  private
    { Private declarations }
    FRenderer: TRenderer;
    FComm: TComm;
    FMapData: TMapData;
    FPlayer: TPlayer;

    procedure OnCommUpdate;
  public
    { Public declarations }
  end;

var
  MainWindowClass: TMainWindowClass;

implementation

{$R *.dfm}

procedure TMainWindowClass.ButtonClick(Sender: TObject);
begin
  // Manuln ovldn
  if not FComm.GameEnd then
    FPlayer.HandleKey(Word(string((Sender as TButton).Caption)[1]));
end;

procedure TMainWindowClass.FormCreate(Sender: TObject);
begin
  // Vytvoen podprnch td
  FRenderer := TRenderer.Create(Image1.Canvas);
  FRenderer.SetViewportSize(Image1.ClientWidth, Image1.ClientHeight);

  FComm := TComm.Create;

  FMapData := TMapData.Create(FComm, FRenderer);

  // Nastaven funkce po aktualizaci ze serveru
  FComm.OnUpdate := OnCommUpdate;
end;

procedure TMainWindowClass.FormDestroy(Sender: TObject);
begin
  // Vymazn td
  if Assigned(FPlayer) then
    FPlayer.Destroy;
  FMapData.Destroy;
  FComm.Destroy;
  FRenderer.Destroy;
end;

procedure TMainWindowClass.FormPaint(Sender: TObject);
begin
  // Vykreslen hry
  FRenderer.ClearScreen;
  FMapData.Render;
end;

procedure TMainWindowClass.FormResize(Sender: TObject);
begin
  // Unused :)
  FRenderer.SetViewportSize(ClientWidth, ClientHeight);
end;

procedure TMainWindowClass.Konec1Click(Sender: TObject);
begin
  // Zave aplikaci
  Close;
end;

procedure TMainWindowClass.Nastavitadresuserveru1Click(Sender: TObject);
var addr: string;
begin
  // Umon zmnit adresu serveru
  addr := InputBox('Adresa serveru', 'Zadejte adresu serveru: ', '');
  if addr <> '' then
    FComm.ServerAddress := addr;
end;

procedure TMainWindowClass.Novhra1Click(Sender: TObject);
begin
  // Vymazn hre
  if Assigned(FPlayer) then
    FPlayer.Destroy;

  // Vytvoen AI hre / normlnho hre
  if useAI.Checked then
    FPlayer := TAIPlayer.Create(FMapData)
  else
    FPlayer := TPlayer.Create(FMapData);

  // Spust hru
  FComm.StartGame;
end;

procedure TMainWindowClass.UpdaterTimer(Sender: TObject);
begin
  // Pekresl obrazovku
  Paint;
end;

procedure TMainWindowClass.OnCommUpdate;
var task: ITask;
begin
  // Po konci hry
  if FComm.GameEnd then
  begin
    // Se vype vsledek a resetuje se dialog
    MessageDlg(FComm.GameResult, mtInformation, [mbOK], 0);
    playerCurrent.Caption := '0';
    playerTotal.Caption := '0';
    cpuCurrent.Caption := '0';
    cpuTotal.Caption := '0';
    movesLeft.Caption := '0';
  end else
  begin
    // Provede se aktualizace mapy, GUI, hre
    FMapData.Update;
    movesLeft.Caption := IntToStr(FComm.MovesLeft);
    playerCurrent.Caption := IntToStr(FComm.Player.Current);
    playerTotal.Caption := IntToStr(FComm.Player.Total);
    if FComm.HasServerPlayer then
    begin
      cpuCurrent.Caption := IntToStr(FComm.ServerPlayer.Current);
      cpuTotal.Caption := IntToStr(FComm.ServerPlayer.Total);
    end;
    cpuBox.Visible := FComm.HasServerPlayer;
    task := TTask.Create(procedure ()
    begin
      Sleep(100);
      try
       FPlayer.OnUpdate;
      finally
      end;
    end
    );
    task.Start;
  end;
end;

end.
