unit MapData;

interface

uses Communication, Renderer, System.SysUtils, Vcl.Graphics;

type
  TMapData = class
    private
      FComm: TComm;
      FRenderer: TRenderer;
      FHasData: Boolean;

      FMovesLeft: Integer;
      FMapData: array[1..10, 1..10] of string;

      function StringToColor(C: string): TColor;
    public
      constructor Create(Comm: TComm; R: TRenderer);
      destructor Destroy; override;

      procedure Update;
      procedure Render;

      function IsTreasureAt(X: Integer; Y: Integer): Boolean;
      function CanMoveToPosition(X: Integer; Y: Integer): Boolean;
      procedure GetHomeXY(var X, Y: Integer);
      procedure Move(Dir: TCommDirection);

      property Comm: TComm read FComm;
  end;

implementation

constructor TMapData.Create(Comm: TComm; R: TRenderer);
begin
  FComm := Comm;
  FRenderer := R;
  FHasData := False;
end;

destructor TMapData.Destroy;
begin
end;

procedure TMapData.Update;
var i, j, n: Integer;
begin
  FMovesLeft := StrToInt(FComm.LineData[1]);
  for i := 2 to 11 do
  begin
    j := 0;
    n := 1;
    while n <= 10 do
    begin
      FMapData[i - 1, n] := FComm.LineData[i].Substring(j, 2);
      Inc(n);
      Inc(j, 2);
    end;
  end;

  FHasData := True;
end;

procedure TMapData.Render;
var i, j: Integer;
    color: TColor;
begin
  if FHasData then
    for i := 0 to 9 do
      for j := 0 to 9 do
      begin
        if (j + 1 = FComm.Player.X) and (i + 1 = FComm.Player.Y) then
          color := clLime
        else
        if FComm.HasServerPlayer and (j + 1 = FComm.ServerPlayer.X)
          and (i + 1 = FComm.ServerPlayer.Y) then
          color := clPurple
        else
          color := StringToColor(FMapData[i + 1, j + 1]);

        FRenderer.RenderQuadText(j * 60, i * 60, 60, 60,
            FMapData[i + 1, j + 1], color)
      end;
end;

function TMapData.StringToColor(C: string): TColor;
begin
  if C = '..' then
    Result := clGray
  else if C = 'XX' then
    Result := clBlack
  else if C = '++' then
    Result := clGreen
  else if C = '--' then
    Result := clRed
  else
    Result := clSilver;
end;

function TMapData.CanMoveToPosition(X: Integer; Y: Integer): Boolean;
var c: String;
begin
  Result := ((X > 0) and (X < 11)) and ((Y > 0) and (Y < 11));
  if Result then
  begin
    c := FMapData[Y, X];
    Result := Result and (c <> 'XX') and (c <> '--');
  end;
  if FComm.HasServerPlayer then
  begin
    Result := Result and not ((X = FComm.ServerPlayer.X) and
      (Y = FComm.ServerPlayer.Y));
  end;
end;

procedure TMapData.Move(Dir: TCommDirection);
begin
  FComm.SendMove(Dir);
end;

function TMapData.IsTreasureAt(X: Integer; Y: Integer): Boolean;
var s: string;
begin
  s := FMapData[Y, X];
  Result := (s <> '..') and (s <> '++') and (s <> '--') and (s <> 'XX');
end;

procedure TMapData.GetHomeXY(var X: Integer; var Y: Integer);
label Skip;
var i, j: Integer;
begin
  for i := 1 to 10 do
    for j := 1 to 10 do
      if FMapData[j, i] = '++' then
      begin
        X := i;
        Y := j;
        goto Skip;
      end;
  Skip:
end;

end.
